<?php
/**
 * Copyright © Firebear Studio, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Firebear\PlatformSap\Model\Export\Adapter\Gateway;

use Firebear\ImportExport\Api\Export\History\CompressInterface;
use Firebear\ImportExport\Model\Filesystem\File\ReadFactory;
use Firebear\ImportExport\Model\Source\Factory as SourceFactory;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Directory\WriteFactory as DirectoryWriteFactory;
use Magento\Framework\Filesystem\File\WriteFactory as FileWriteFactory;
use Magento\Framework\Stdlib\DateTime\Timezone;
use Firebear\PlatformSap\Model\LogRepository;

/**
 * Sap abstract gateway
 */
class AbstractGateway extends \Firebear\ImportExport\Model\Source\Type\Rest
{
    /**
     * @var array
     */
    private $behaviorData = [];

    /**
     * @var LogRepository
     */
    private $logRepository;

    /**
     * @var array
     */
    private $config;

    const STATUS_SUCCESS = 2;
    const STATUS_ERROR = 1;

    /**
     * AbstractGateway constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param Filesystem $filesystem
     * @param ReadFactory $readFactory
     * @param DirectoryWriteFactory $writeFactory
     * @param FileWriteFactory $fileWrite
     * @param Timezone $timezone
     * @param SourceFactory $factory
     * @param CacheInterface $cache
     * @param LogRepository $logRepository
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        Filesystem $filesystem,
        ReadFactory $readFactory,
        DirectoryWriteFactory $writeFactory,
        FileWriteFactory $fileWrite,
        Timezone $timezone,
        SourceFactory $factory,
        CacheInterface $cache,
        LogRepository  $logRepository,
        CompressInterface $compressCommand
    ) {
        parent::__construct(
            $scopeConfig,
            $filesystem,
            $readFactory,
            $writeFactory,
            $fileWrite,
            $timezone,
            $factory,
            $cache,
            $compressCommand
        );
        $this->logRepository = $logRepository;
    }


    /**
     * @param string $data Json request body from Job's settings
     * @param null|string $type
     * @param null $format
     *
     * @return mixed json-structured data
     */
    public function convertToFormat($data, $type = null, $format = null)
    {
        $data = [];
        $data = json_encode($data);
        return parent::convertToFormat($data, $type, $format);
    }

    /**
     * @param $data
     * @return array
     * @throws \Magento\Framework\Exception\CouldNotSaveExceptions
     */
    public function uploadSourceData($data)
    {
        $result = [];
        $url = 'http://%s:%s/B1iXcellerator/exec/ipo/vP.%s.in_HCSX/com.sap.b1i.vplatform.runtime/'.
            '%s/%s.ipo/proc/%s';
        if (!empty($data['store'])) {
            $config = $this->getConfig('store', $data['store']);
        } else {
            $config = $this->getConfig();
        }

        $log = $this->logRepository->createByData($data);

        $sourceData = $this->_getSourceClient()->execute(
            sprintf(
                $url,
                $config['sapServerIp'],
                $config['sapServerPort'],
                $config['sapSystemId'],
                $config['sapOrderIpo'],
                $config['sapOrderIpo'],
                static::SAP_ENTITY_ENDPOINT
            ),
            'POST',
            json_encode($data),
            ['Content-Type' => 'application/json']
        );

        if ($sourceData && 200 == $sourceData->info->http_code) {
            $message = __(
                'The entity %1 was successfully imported to the Sap',
                (isset($data['increment_id'])) ? $data['increment_id'] : $data['entity_id']
            );
            $this->logRepository->updateByData($log, [
                'message' => $message,
                'status' => self::STATUS_SUCCESS
            ]);
            $result['error'] = false;
        } else {
            $message = __(
                'The entity %1 not exported to the Sap. Message: %2',
                [
                    (isset($data['increment_id'])) ? $data['increment_id'] : $data['entity_id'],
                    strip_tags(str_replace('<', ' <', $sourceData->response))
                ]
            );
            $this->logRepository->updateByData($log, [
                'message' => $message,
                'status' => self::STATUS_ERROR
            ]);
            $result['error'] = true;
        }

        $response = @simplexml_load_string($sourceData->response);
        if ($response) {
            $response = json_decode(json_encode($response), true);
            if (isset($response['object_id'])) {
                $result['object_id'] = $response['object_id'];
            }
        }

        $result['message'] = $message;

        return $result;
    }

    /**
     * @return array
     */
    protected function getConfig($scopeType = 'default', $scope = null)
    {
        if (empty($this->config)) {

            $behaviorData = $this->getBehaviorData();

            if (!empty($behaviorData)) {
                $this->config = [
                    "sapServerIp" => \trim($behaviorData['sap_server_ip']),
                    "sapServerPort"     => \trim($behaviorData['sap_server_port']),
                    "sapSystemId"  => \trim($behaviorData['sap_system_id']),
                    "sapOrderIpo" => \trim($behaviorData['sap_order_ipo']),
                ];
            } else {
                $this->config = [
                    "sapServerIp" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_server_ip', $scopeType, $scope)
                    ),
                    "sapServerPort" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_server_port', $scopeType, $scope)
                    ),
                    "sapSystemId" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_system_id', $scopeType, $scope)
                    ),
                    "sapOrderIpo" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_order_ipo', $scopeType, $scope)
                    ),
                ];
            }
        }
        return $this->config;
    }

    /**
     * Set behavior data
     *
     * @param $data
     *
     * @return \Firebear\Platformsap\Model\Export\Adapter\Gateway\AbstractGateway
     */
    public function setBehaviorData($data)
    {
        $this->behaviorData = $data;

        return $this;
    }

    /**
     * Get behavior data
     *
     * @return array
     */
    public function getBehaviorData()
    {
        return $this->behaviorData;
    }
}
