<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model\Export\Adapter\Gateway;

use Firebear\ImportExport\Api\Export\History\CompressInterface;
use Firebear\ImportExport\Logger\Logger;
use Firebear\ImportExport\Model\Filesystem\File\ReadFactory;
use Firebear\ImportExport\Model\Source\Factory as SourceFactory;
use Firebear\ImportExport\Traits\General as GeneralTrait;
use Firebear\PlatformSap\Model\LogRepository;
use Magento\Customer\Model\ResourceModel\AddressRepository;
use Magento\Customer\Model\ResourceModel\CustomerRepository;
use Magento\Eav\Model\ResourceModel\Entity\Attribute;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Directory\WriteFactory as DirectoryWriteFactory;
use Magento\Framework\Filesystem\File\WriteFactory as FileWriteFactory;
use Magento\Framework\Stdlib\DateTime\Timezone;
use Symfony\Component\Console\Output\ConsoleOutput;
use Magento\Framework\ObjectManagerInterface;
use Magento\Store\Model\StoreManagerInterface as StoreManager;

/**
 * Sap customer gateway
 */
class Customer extends AbstractGateway
{
    use GeneralTrait;

    const SAP_ENTITY_ENDPOINT = 'BusinessPartner/Create';

    /**
     * Varchar attribute values table name
     *
     * @var string
     */
    protected $_attributeTable = 'customer_entity_varchar';

    /**
     * @var CustomerRepository
     */
    protected $customerRepository;

    /**
     * @var AddressRepository
     */
    private $addressRepository;

    /**
     * @var ResourceConnection
     */
    protected $resource;

    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    protected $connection;

    /**
     * @var Attribute
     */
    protected $eavAttribute;

    /**
     * @var Attribute
     */
    protected $objectManager;

    /**
     * @var StoreManager
     */
    protected $storeManager;

    /**
     * Customer constructor.
     *
     * @param ScopeConfigInterface   $scopeConfig
     * @param CustomerRepository     $customerRepository
     * @param AddressRepository      $addressRepository
     * @param Logger                 $logger
     * @param ConsoleOutput          $output
     * @param Filesystem             $filesystem
     * @param ReadFactory            $readFactory
     * @param DirectoryWriteFactory  $writeFactory
     * @param FileWriteFactory       $fileWrite
     * @param Timezone               $timezone
     * @param SourceFactory          $factory
     * @param CacheInterface         $cache
     * @param LogRepository          $logRepository
     * @param ResourceConnection     $resource
     * @param Attribute              $eavAttribute
     * @param CompressInterface      $compressCommand
     * @param ObjectManagerInterface $objectManager
     * @param StoreManager $storeManager
     *
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        CustomerRepository $customerRepository,
        AddressRepository $addressRepository,
        Logger $logger,
        ConsoleOutput $output,
        Filesystem $filesystem,
        ReadFactory $readFactory,
        DirectoryWriteFactory $writeFactory,
        FileWriteFactory $fileWrite,
        Timezone $timezone,
        SourceFactory $factory,
        CacheInterface $cache,
        LogRepository $logRepository,
        ResourceConnection $resource,
        Attribute $eavAttribute,
        CompressInterface $compressCommand,
        ObjectManagerInterface $objectManager,
        StoreManager $storeManager
    ) {
        parent::__construct(
            $scopeConfig,
            $filesystem,
            $readFactory,
            $writeFactory,
            $fileWrite,
            $timezone,
            $factory,
            $cache,
            $logRepository,
            $compressCommand
        );
        $this->customerRepository = $customerRepository;
        $this->addressRepository = $addressRepository;
        $this->resource = $resource;
        $this->connection = $resource->getConnection();
        $this->_logger = $logger;
        $this->eavAttribute = $eavAttribute;

        if (empty($this->_logger->getFileName())) {
            $this->_logger->setFileName('sapCustomerUpdate');
        }

        $this->output = $output;
        $this->objectManager = $objectManager;
        $this->storeManager = $storeManager;
    }

    /**
     * @param $data
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function exportCustomer($data)
    {
        $result = [];
        $customerRegistry = $this->objectManager->create(\Magento\Customer\Model\CustomerRegistry::class);

        if ($data['entity'] == 'customer') {
            $customer = $customerRegistry->retrieveByEmail($data['email']);
            $result = $this->addCustomer($customer->getDataModel(), $data);
        } elseif ($data['entity'] == 'customer_address') {
            $address = $this->addressRepository->getById($data['_entity_id']);
            $customerId = $address->getCustomerId();
            $customer = $customerRegistry->retrieve($customerId);
            $result = $this->addCustomer($customer->getDataModel(), $data);
        }

        return $result;
    }

    /**
     * @param \Magento\Customer\Api\Data\CustomerInterface $customer
     * @param array $data
     */
    public function addCustomer(\Magento\Customer\Api\Data\CustomerInterface $customer, $data = [])
    {
        $sapEntityData['entity'] = 'customer';
        $sapEntityData['entity_id'] = (int) $customer->getId();
        $sapEntityData['firstname'] = $customer->getFirstname();
        $sapEntityData['lastname'] = $customer->getLastname();
        $sapEntityData['email'] = $customer->getEmail();
        $sapEntityData['taxvat'] = $customer->getTaxvat();

        $sapId = $customer->getCustomAttribute('sap_id');
        if (!empty($sapId)) {
            $sapId = $sapId->getValue();
        }
        $sapEntityData['sap_id'] = $sapId;

        if (empty($customer->getAddresses())) {
            $customer = $this->customerRepository->getById($customer->getId());
        }

        foreach ($customer->getAddresses() as $customerAddress) {
            $billingAddress = $this->prepareAddress($customerAddress, 'billing');
            $shippingAddress = $this->prepareAddress($customerAddress, 'shipping');

            if (empty($sapEntityData['taxvat']) && !empty($billingAddress['vat_id'])) {
                $sapEntityData['taxvat'] = $billingAddress['vat_id'];
            }

            if (!empty($billingAddress)) {
                $sapEntityData['addresses'][] = $billingAddress;
            }
            if (!empty($shippingAddress)) {
                $sapEntityData['addresses'][] = $shippingAddress;
            }
            if ($customerAddress->isDefaultBilling()) {
                $sapEntityData['phone'] = $customerAddress->getTelephone();
            }
            if ($customerAddress->isDefaultShipping()) {
                $sapEntityData['phone1'] = $customerAddress->getTelephone();
            }
        }

        $store = $this->storeManager->getStore($customer->getStoreId());
        $sapEntityData['bp_code'] = \trim(
            $this->scopeConfig->getValue(
                'firebear_importexport/sap/sap_default_bp_prefix',
                'store',
                $store->getCode()
            )
        );

        $updateResponse = $this->uploadSourceData($sapEntityData);

        if (!$updateResponse['error']) {
            if (isset($updateResponse['object_id'])) {
                $customer->setCustomAttribute(
                    'sap_id',
                    $updateResponse['object_id']
                );
                $this->updateSapInternalId($updateResponse['object_id'], $customer->getId());
            }
            $this->addLogWriteln($updateResponse['message'], $this->output);
        } else {
            $this->addLogWriteln($updateResponse['message'], $this->output, 'error');
        }

        return $updateResponse;
    }

    /**
     * @param $sapInternalId
     * @param $customerId
     */
    private function updateSapInternalId($sapInternalId, $customerId)
    {
        $attributeId = $this->eavAttribute->getIdByCode('customer', 'sap_id');
        $this->connection->insertOnDuplicate(
            $this->resource->getTableName(
                $this->_attributeTable
            ),
            [
                'attribute_id' => $attributeId,
                'entity_id' => $customerId,
                'value' => $sapInternalId
            ],
            ['attribute_id', 'entity_id', 'value']
        );
    }

    /**
     * @param \Magento\Customer\Api\Data\AddressInterface $customerAddress
     * @param $type
     * @return mixed
     */
    protected function prepareAddress(\Magento\Customer\Api\Data\AddressInterface $customerAddress, $type)
    {
        $sapId = $customerAddress->getCustomAttribute('sap_id');
        if (!empty($sapId)) {
            $sapId = $sapId->getValue();
            if ($type == 'billing' && !$customerAddress->isDefaultBilling()) {
                return [];
            } elseif ($type == 'billing') {
                if (strpos($sapId, 'Ship') !== false) {
                    $sapId = str_replace('Ship', "Bill", $sapId);
                }
            }
            if ($type == 'shipping' && !$customerAddress->isDefaultShipping()) {
                return [];
            } elseif ($type == 'shipping') {
                if (strpos($sapId, 'Bill') !== false) {
                    $sapId = str_replace('Bill', "Ship", $sapId);
                }
            }
        }
        $address['entity_id'] = (int) $customerAddress->getId();
        $address['sap_id'] = $sapId;
        $address['postcode'] = $customerAddress->getPostcode();
        $address['city'] = $customerAddress->getCity();
        $street = $customerAddress->getStreet();
        $address['street_0'] = isset($street[0]) ? $street[0] : '';
        $address['street_1'] = isset($street[1]) ? $street[1] : '';
        $address['country_id'] = $customerAddress->getCountryId();
//        @TODO map SAP State with Magento region_id
//        $address['region_id'] = substr($customerAddress->getRegion()->getRegionCode(), 0, 2);
        $addressRegion = $customerAddress->getRegion();
        if (!empty($addressRegion)) {
            $address['region_code'] = $customerAddress->getRegion()->getRegionCode();
        }

        $address['vat_id'] = $customerAddress->getVatId();
        $address['telephone'] = $customerAddress->getTelephone();
        $address['type'] = $type;
        $address['default'] = 0;

        if ($type == 'billing' && $customerAddress->isDefaultBilling()) {
            $address['default'] = 1;
        }
        if ($type == 'shipping' && $customerAddress->isDefaultShipping()) {
            $address['default'] = 1;
        }

        return $address;
    }
}
