<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model\Import;

use Firebear\ImportExport\Model\Import\Context;
use Firebear\ImportExport\Model\Import\ImportAdapterInterface;
use Firebear\ImportExport\Traits\Import\Entity as ImportTrait;
use InvalidArgumentException;
use Magento\Eav\Model\Config;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Stdlib\StringUtils;
use Magento\ImportExport\Model\Import;
use Magento\Indexer\Model\Indexer\StateFactory;
use Magento\ImportExport\Model\Import\Entity\AbstractEntity as AbstractEntity;
use Magento\Catalog\Model\ProductFactory;
use Magento\Framework\Json\Helper\Data as JsonHelper;
use Firebear\ImportExport\Helper\Data as ImportHelper;

/**
 * StockSourceQty Import
 */
class CustomerGroup extends AbstractEntity implements ImportAdapterInterface
{
    use ImportTrait;

    /**
     * Keys Which Used To Build Result Data Array For Future Update
     */
    const ENTITIES_TO_CREATE_KEY = 'entities_to_create';

    const ENTITIES_TO_UPDATE_KEY = 'entities_to_update';

    /**
     * Main Table Name
     *
     * @var string
     */
    protected $_mainTable = 'customer_group';

    /**
     * @var ResourceConnection
     */
    protected $resource;

    /**
     * @var ImportHelper
     */
    protected $_helper;

    /**
     * Entity columns
     *
     * @var string[]
     */
    protected $_importFields = [
        'customer_group_code',
        'tax_class_id',
        'sap_id'
    ];

    /**
     * CustomerGroup constructor.
     * @param Context $context
     * @param JsonHelper $jsonHelper
     * @param \Magento\ImportExport\Model\ResourceModel\Import\Data $importData
     * @param Config $config
     * @param ResourceConnection $resource
     * @param StringUtils $string
     * @param ImportHelper $helper
     */
    public function __construct(
        Context $context,
        JsonHelper $jsonHelper,
        \Magento\ImportExport\Model\ResourceModel\Import\Data $importData,
        Config $config,
        ResourceConnection $resource,
        StringUtils $string,
        ImportHelper $helper
    ) {
        $this->_logger = $context->getLogger();
        $this->output = $context->getOutput();
        $this->jsonHelper = $jsonHelper;
        $this->_importExportData = $context->getImportExportData();
        $this->_resourceHelper = $context->getResourceHelper();
        $this->string = $string;
        $this->errorAggregator = $context->getErrorAggregator();
        $this->_dataSourceModel =  $context->getDataSourceModel();
        $this->resource = $resource;
        $this->_connection = $resource->getConnection();
        $this->_helper = $helper;
    }

    /**
     * Retrieve All Fields Source
     *
     * @return array
     */
    public function getAllFields()
    {
        return $this->_importFields;
    }

    /**
     * Import Data Rows
     *
     * @return boolean
     */
    protected function _importData()
    {
        while ($bunch = $this->_dataSourceModel->getNextBunch()) {
            $toCreate = [];
            $toUpdate = [];
            $toDelete = [];
            foreach ($bunch as $rowNumber => $rowData) {

                /* validate data */
                if (!$rowData || !$this->validateRow($rowData, $rowNumber)) {
                    continue;
                }

                if ($this->getErrorAggregator()->hasToBeTerminated()) {
                    $this->getErrorAggregator()->addRowToSkip($rowNumber);
                    continue;
                }

                /* behavior selector */
                switch ($this->getBehavior()) {
                    case Import::BEHAVIOR_DELETE:
                        $toDelete[] = $rowData['customer_group_code'];
                        break;
                    case Import::BEHAVIOR_REPLACE:
                        $data = $this->_prepareDataForReplace($rowData);
                        $toUpdate = array_merge($toUpdate, $data[self::ENTITIES_TO_UPDATE_KEY]);
                        break;
                    case Import::BEHAVIOR_APPEND:
                        $data = $this->_prepareDataForUpdate($rowData);
                        $toCreate = array_merge($toCreate, $data[self::ENTITIES_TO_CREATE_KEY]);
                        $toUpdate = array_merge($toUpdate, $data[self::ENTITIES_TO_UPDATE_KEY]);
                        break;
                }
                $this->_processedEntitiesCount++;
            }
            /* save prepared data */
            if ($toCreate || $toUpdate) {
                $this->_saveEntities($toCreate, $toUpdate);
            }
            if ($toDelete) {
                $this->_deleteEntities($toDelete);
            }
        }
        return true;
    }

    /**
     * Delete List Of Entities
     *
     * @param array $toDelete Entities Id List
     * @return $this
     */
    protected function _deleteEntities(array $toDelete)
    {
        $condition = $this->_connection->quoteInto(
            'customer_group_code IN (?)',
            $toDelete
        );
        $this->_connection->delete($this->getMainTable(), $condition);

        return $this;
    }

    /**
     * Update And Insert Data In Entity Table
     *
     * @param array $toCreate Rows for insert
     * @param array $toUpdate Rows for update
     * @return $this
     */
    protected function _saveEntities(array $toCreate, array $toUpdate)
    {
        if ($toCreate) {
            $this->_connection->insertMultiple(
                $this->getMainTable(),
                $toCreate
            );
        }
        if ($toUpdate) {
            $this->_connection->insertOnDuplicate(
                $this->getMainTable(),
                $toUpdate,
                $this->_getEntityFieldsToUpdate($toUpdate)
            );
        }
        return $this;
    }

    /**
     * Filter The Entity That Are Being Updated So We Only Change Fields Found In The Importer File
     *
     * @param array $toUpdate
     * @return array
     */
    protected function _getEntityFieldsToUpdate(array $toUpdate)
    {
        $firstEntity = reset($toUpdate);
        $columnsToUpdate = array_keys($firstEntity);
        $fieldsToUpdate = array_filter(
            $this->getAllFields(),
            function ($field) use ($columnsToUpdate) {
                return in_array($field, $columnsToUpdate);
            }
        );
        return $fieldsToUpdate;
    }

    /**
     * Prepare Data For Update
     *
     * @param array $rowData
     * @return array
     */
    protected function _prepareDataForUpdate(array $rowData)
    {
        $toCreate = [];
        $toUpdate = [];

        $newEntity = false;
        $customerGroupId = $this->_isExistEntity($rowData);
        if (empty($customerGroupId)) {
            $newEntity = true;
            $entityRow = [
                'customer_group_code' => $rowData['customer_group_code']
            ];
        } else {
            $entityRow = [
                'customer_group_id' => $customerGroupId,
                'customer_group_code' => $rowData['customer_group_code']
            ];
        }

        /* prepare data */
        $entityRow = $this->_prepareEntityRow($entityRow, $rowData);
        if ($newEntity) {
            $toCreate[] = $entityRow;
        } else {
            $toUpdate[] = $entityRow;
        }
        return [
            self::ENTITIES_TO_CREATE_KEY => $toCreate,
            self::ENTITIES_TO_UPDATE_KEY => $toUpdate
        ];
    }

    /**
     * Check If Entity Is Present In Database
     *
     * @param array $rowData
     * @return bool|int
     */
    protected function _isExistEntity(array $rowData)
    {
        $bind = [':customer_group_code' => $rowData['customer_group_code']];
        /** @var $select \Magento\Framework\DB\Select */
        $select = $this->_connection->select();
        $select->from($this->getMainTable(), 'customer_group_id')
            ->where('customer_group_code = :customer_group_code');

        return $this->_connection->fetchOne($select, $bind);
    }

    /**
     * Retrieve Main Table Name
     *
     * @return string
     */
    public function getMainTable()
    {
        return $this->resource->getTableName(
            $this->_mainTable
        );
    }

    /**
     * Prepare Data For Replace
     *
     * @param array $rowData
     * @return array
     */
    protected function _prepareDataForReplace(array $rowData)
    {
        $toUpdate = [];
        $entityRow = [
            'customer_group_code' => $rowData['customer_group_code']
        ];
        /* prepare data */
        $toUpdate[] = $this->_prepareEntityRow($entityRow, $rowData);
        return [
            self::ENTITIES_TO_UPDATE_KEY => $toUpdate
        ];
    }

    /**
     * Prepare Entity Field Values
     *
     * @param array $entityRow
     * @param array $rowData
     * @return array
     */
    protected function _prepareEntityRow(array $entityRow, array $rowData)
    {
        $keys = array_keys($entityRow);
        foreach ($this->getAllFields() as $field) {
            if (!in_array($field, $keys) && isset($rowData[$field])) {
                $entityRow[$field] = $rowData[$field];
            }
        }
        return $entityRow;
    }


    /**
     * Save Validated Bunches
     *
     * @return $this
     * @throws LocalizedException
     */
    protected function _saveValidatedBunches()
    {
        $source = $this->_getSource();
        $currentDataSize = 0;
        $bunchRows = [];
        $startNewBunch = false;
        $nextRowBackup = [];
        $maxDataSize = $this->_resourceHelper->getMaxDataSize();
        $bunchSize = $this->_importExportData->getBunchSize();

        $source->rewind();
        $this->_dataSourceModel->cleanBunches();
        $file = null;
        $jobId = null;
        if (isset($this->_parameters['file'])) {
            $file = $this->_parameters['file'];
        }
        if (isset($this->_parameters['job_id'])) {
            $jobId = $this->_parameters['job_id'];
        }

        while ($source->valid() || $bunchRows) {
            if ($startNewBunch || !$source->valid()) {
                $this->_dataSourceModel->saveBunches(
                    $this->getEntityTypeCode(),
                    $this->getBehavior(),
                    $jobId,
                    $file,
                    $bunchRows
                );
                $bunchRows = $nextRowBackup;
                $currentDataSize = strlen($this->jsonHelper->jsonEncode($bunchRows));
                $startNewBunch = false;
                $nextRowBackup = [];
            }

            if ($source->valid()) {
                try {
                    $rowData = $source->current();
                } catch (InvalidArgumentException $e) {
                    $this->addRowError($e->getMessage(), $this->_processedRowsCount);
                    $this->_processedRowsCount++;
                    $source->next();
                    continue;
                }
                $rowData = $this->customBunchesData($rowData);
                $this->_processedRowsCount++;
                if ($this->validateRow($rowData, $source->key())) {
                    $rowSize = strlen($this->jsonHelper->jsonEncode($rowData));

                    $isBunchSizeExceeded = $bunchSize > 0 && count($bunchRows) >= $bunchSize;

                    if ($currentDataSize + $rowSize >= $maxDataSize || $isBunchSizeExceeded) {
                        $startNewBunch = true;
                        $nextRowBackup = [$source->key() => $rowData];
                    } else {
                        $bunchRows[$source->key()] = $rowData;
                        $currentDataSize += $rowSize;
                    }
                }
                $source->next();
            }
        }

        foreach ($this->getErrorAggregator()->getAllErrors() as $error) {
            $this->addLogWriteln(
                __('%1 in row %2', $error->getErrorMessage(), $error->getRowNumber()),
                $this->getOutput()
            );
        }
        return $this;
    }

    /**
     * EAV entity type code getter.
     *
     * @return string
     */
    public function getEntityTypeCode()
    {
        return 'customer_group';
    }

    /**
     * Validate data row
     *
     * @param array $rowData
     * @param int $rowNum
     * @return boolean
     */
    public function validateRow(array $rowData, $rowNum)
    {
        $result = true;
        foreach ($this->_importFields as $field) {
            if (isset($rowData[$field]) &&
                $rowData[$field] === ''
            ) {
                $this->addRowError(__('Invalid %1 value', $field), $rowNum);
                $result = false;
            }
        }

        if (!$result) {
            return $result;
        }
        return $result;
    }
}
